@@ -14,62 +14,12 @@ from fmqlutils.cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator,
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent, muBVC
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVCOfSTProp, refsOfST
 
+# ..webReportUtils
 from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce4, flattenPropValues 
-from fmqlreports.webReportUtils import muPlotRef, makePlots, vistasOfVISNByOne
 
-"""
-NEXT ON CONTENT (not on visuals):
-
-- Filler breakdown ala Placer (no mixing for now)
-- may only do completed ie/ nix discontinued/cancelled for now
-- expose remote site service name for placers? ie/ to link em up?
-    ifc_remote_service_name (pair with local services so see if 1 remote for > 1 local?)
-- visn named (not just other visns)
-- filler only patients (delay reg?) ... what?
-- WILL START with COMPLETED vs OTHER for all CONSULTs and then just do COMPLETED in rest of report.
-- locations:
-    from/... location 44 (no pattern seen per see but do they all have similar workloads?)
-    ordering_facility (is precise to clinic)
-
-Portland vs Pug as big (can do TeleMental as part of filler breaks):
-... TODO:
-    - TeleMental highlight for POR from all four others?
-    - setup BOI for Pharmacy the same
-- portland has more filler only services
-- portland has more placer consults
-- pug has more filler only patients (because proportionately more FILLER IFCs ie/ few placers)
-- their user proportions are remarkedly similar
-- common placers (50% PUG, 76% POR)
-- spec placers (20 PUG, 5 POR)
-  - for PUG, POR dominates, for POR, BOISE dominates
-  - PUG to POR is for TELEMENTAL/ TELE NEURO! WWW does this the most for POR too and SPO has it too [TELEMENTAL is POR's second highest FILLER after TELER ie/ POR specializes in this]
-  - POR to BOI (smaller amount) is Pharmacy! [NEED TO SEE HOW MUCH PHARMACY and not just TELER that BOI does]
-  - lessor spec placees differ:
-    - CANCER TEAM from PUG (expected!) but not POR
-  - few other VISNs ... no real pattern
-  
-FORMATTING IMPROVEMENTS <--- key for going forward
-- 1. MOVE all calcs to "Data Prep/Frame Phase" ... also key stats (vs so many details) out
-- 2. Consider plots separate ie/ can gen first and then write blurbs ... could cut down on data needed for actual text/paragraphs ... could do tables in this phase too ie/ table of DF vs Graph of DF ... plot from frames/table from frames
-- 3. Better struct for md ... add section, add header, add paragraph, add table ...
-"""
-
-"""
-Next week should be last on IFCs (for now): it'll add Puget and flesh out the findings for the other three VistAs (the intra and inter VISN aspects as well as the Filler as opposed to the Placers of IFCs).
-            	+
-Consider how to see range of test consults in B1930 [ie how to feed into that]
-"""
-
-""" 
-TODO NEXT: for basic, do filler 
+"""   
+TODO NEXT: 
 <----- goal is finish IFC in three pushes along with TeleR (into 2 of 3) ... deliver mid Aug
-- travel: break to VISN out and shorten table
-- filler:
-  - VIRS, DISCHARGE FOLLOW UP, TRAVELLING VETERAN, OTHER (do simply for now to parallel)
-  ... then try PUG ... and will flip FILLER/PLACER on Volume
-- ? DISCHARGE follow up in FILLER -- what is it on the other end? as not obvious
-  ... add note TODO now
-- OTHER Q: if high # local -- is there a type of things ALWAYS DONE LOCALLY?
 - go to DF (before doing services more break etc) ... typerDFPlay
 - break out other services
   - one reason for so many placer services is 'embeds destination' => for any consult type, need where as well (any exs of 'type for x', 'type for y'? ex/ ENDOCRINOLOGY (visit and non visit) ... POR and SEATTLE ... UROLOGY is another example ... with many many. see if can unify using formal meta? ie disciplines, smaller is PODIATRY but still over three centers
@@ -124,7 +74,8 @@ TO WORK FILLER SEPARATION:
 
 <----------- [1] come back here and to [2] 2005 rep for below (XRAY GROUP == group of any etc) before going back + [3] We're going to add the edge case to our simulation data: spec/proc to two readers and see that both see work for the other. Mike saw that the TeleReader displays the "unintended" in a particular way. More on that next week.
 """
-def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Images", doDAD=False):
+
+def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Images"):
 
     type123YR1, sts123YR1All = splitTypeDatas(stationNo, "123", reductionLabel="YR1", expectSubTypeProperties=["cprs_status", "routing_facility", "to_service"])
     sts123YR1 = []
@@ -135,21 +86,35 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
             print("** No service for {} ({:,}) so skipping".format(st["_subTypeId"], st["_total"])) # one in POR
             continue
         sts123YR1.append(st)
-                
+        
+    allTTL = sum(st["_total"] for st in sts123YR1)
+    allServices = set(singleValue(st, "to_service") for st in sts123YR1)
+    allUsers = set()
+    allPatients = set()
+    for st in sts123YR1:
+        allUsers |= refsOfST(st, "200")
+        allPatients |= refsOfST(st, "2")
+    if len(allPatients) != type123YR1["patient_name"]["rangeCount"]:
+        raise Exception("Check on patient refs - shouldn't be necessary - checking typer utils")
+        
     institByIEN = reduce4(stationNo)
+
+    # Service overlap with RF may happen due to filler being used in RF 
+    # and WORF consults
+    stWORF = [st for st in sts123YR1 if "routing_facility" not in st]    
+    worfTTL = sum(st["_total"] for st in stWORF)
+    worfServices = set(singleValue(st, "to_service") for st in stWORF)
+    worfUsers = set(user for st in stWORF for user in refsOfST(st, "200"))
+    worfPatients = set(patient for st in stWORF for patient in refsOfST(st, "2"))
             
     # WRF - with routing facility => IFC ie/ wrf <=> ifc
     stWRF = [st for st in sts123YR1 if "routing_facility" in st]
-    placerServicesCheck = set()
     # If routing_facility => ifc_role and vica_versa
     if sum(1 for st in stWRF if "ifc_role" not in st and "routing_facility" in st) or sum(1 for st in sts123YR1 if "ifc_role" in st and "routing_facility" not in st):
         raise Exception("Where routing facility expect role and where role, expect routing facility")
     try: # either placer or filler within st gathered by service, routing_facility
         for st in stWRF:
-            ifcRole = singleValue(st, "ifc_role")
-            # bonus - record placer service for below
-            if ifcRole == "P:PLACER":
-                placerServicesCheck.add(singleValue(st, "to_service"))
+            singleValue(st, "ifc_role")
     except:
         print(json.dumps(st, indent=4))
         raise Exception("> 1 ifc_role in routing_facility'ed ST. Unexpected")
@@ -157,44 +122,7 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
     for st in stWRF: # note: for filler/local overlaps will only gather the filler st's
         service = singleValue(st, "to_service")
         byServiceWRF[service].append(st)
-
-    # Service overlap with RF may happen due to filler being used in RF 
-    # and WORF consults
-    #
-    # REVISIT: handles case where Placer service DOES not get reused in WORF
-    # (enforced below) BUT Portland has DISCONTINUED case where this happens.
-    # ... perhaps discontinued happens before IFC info added to a Consult? 
-    # ... TODO: follow up as not counting this now!
-    # 
-    stWORF = [st for st in sts123YR1 if not ("routing_facility" in st or singleValue(st, "to_service") in placerServicesCheck)]  
-    # Just for tracing -- the WORF Placers
-    # ... PENDING is fine (but rare); DISCONTINUED seems to be too and then WCO has
-    # an Active that has no further action beyond CPRS submit and just sits there.
-    stWORFPlacer = [st for st in sts123YR1 if "routing_facility" not in st and singleValue(st, "to_service") in placerServicesCheck]
-    if len(stWORFPlacer):
-        for st in stWORFPlacer: # POR (DISCONTINUED), PUG (PENDING)
-            stat = singleValue(st, "cprs_status")
-            if not re.search(r'(ACTIVE|DISCONTINUED|PENDING)', stat):
-                print(json.dumps(st, indent=4))
-                raise Exception("If a Placer Service ST lacks Route Info, expect it to be (quickly?) discontinued or to be pending")
-        print("** Warning: {:,} consults were routeless to Placer Services.".format(
-            sum(st["_total"] for st in stWORFPlacer)
-        ))
-    worfTTL = sum(st["_total"] for st in stWORF)
-    worfServices = set(singleValue(st, "to_service") for st in stWORF)
-    worfUsers = set(user for st in stWORF for user in refsOfST(st, "200"))
-    worfPatients = set(patient for st in stWORF for patient in refsOfST(st, "2"))
-    
-    # if WORF removed a placer, must a/c for that so only doing ALL here.
-    stAll = stWORF + stWRF 
-    allTTL = sum(st["_total"] for st in stAll)
-    allServices = set(singleValue(st, "to_service") for st in stAll)
-    allUsers = set()
-    allPatients = set()
-    for st in stAll:
-        allUsers |= refsOfST(st, "200")
-        allPatients |= refsOfST(st, "2")
-                
+        
     # Big: one role per service ie/ placer or filer in practice
     wrfTTL = 0
     wrfUsers = set()
@@ -245,10 +173,6 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
                 # multiple places.
                 raise Exception("Expected one and only one RF for Placer Services")
                 
-    if placerServices != placerServicesCheck:
-        print(len(placerServices))
-        print(len(placerServicesCheck))
-        raise Exception("Placers analyzed before to avoid WORF problems above - make sure in Sync") # TMP til tackle the DISCONTINUED
     if (placerTTL + fillerTTL + worfTTL) != allTTL:
         raise Exception("Totalling mistake - consult totals are exclusive")
     # ONLY for no overlap placer and WORF ... filler may be reused so there can be
@@ -267,6 +191,7 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
     
     # DFNOTE: other to break further -- NEUROSURGERY X, (gather all) -- see if any subtyping done formally?
     # ... DOMICIALLARY REFERRAL (https://www.visn2.va.gov/VISN2/bh/inpatient.asp)
+    # ... VIRS as separate?
     # ... TELEDERMATOLOGY ... why not in TELE... top one WWW to PUG
     # ... granularity of subcategory advised ... how to group so can assign?
     # ... do as part of typer upgrade for categories as types => limited # (can
@@ -274,8 +199,6 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
                 
     # Cache data on Image
     meta = metaOfVistA(stationNo)
-    
-    dad = DumpAndAnalyzeData(stationNo, meta["cutDate"]) if doDAD else None
                         
     # ######################## MU Consults Per ####################
     
@@ -337,7 +260,8 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
         ]
     }
     plotRef = muPlotRef(
-        plotData["allConsults"],
+        plotData["allConsults"]["plotName"], 
+        plotData["allConsults"]["plotMethod"],
         imageDir
     )
     
@@ -376,7 +300,8 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
         ]           
     }
     plotRef = muPlotRef(
-        plotData["allServices"],
+        plotData["allServices"]["plotName"], 
+        plotData["allServices"]["plotMethod"],
         imageDir
     )
     
@@ -385,7 +310,6 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
         plotRef
     )
     
-    # This one differs between Placer and Filler VistAs
     plotData["patientsAndUsers"] = {
         "title": "Users and Patients",
         "plotName": "ifcPatientsAndUsers",
@@ -408,95 +332,54 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
         ]            
     }
     plotRef = muPlotRef(
-        plotData["patientsAndUsers"],
+        plotData["patientsAndUsers"]["plotName"], 
+        plotData["patientsAndUsers"]["plotMethod"],
         imageDir
     )
     blurb = """As seen below, most patients with consults only have local consults and the majority of the balance have both local consults and IFCs (\"mixed\").
     
-"""
-    if placerTTL > fillerTTL:
-        blurb += """A small minority of patients have only fulfilled consults. With fulfilled consults, a patient's home record is typically elsewhere so you would expect to see more such patients and less mixed patients. The lower number of such \"remote-only patients\" should be examined further. 
+A small minority of patients have only fulfilled consults. With fulfilled consults, a patient's home record is typically elsewhere so you would expect to see more such patients and less mixed patients. The lower number of such \"remote-only patients\" should be examined further. 
 
 As for users, a smaller proportion are only involved in local consults. While no users only fulfill consults submitted from elsewhere, a relatively large set only place consults elsewhere. This set of \"placer only users\" needs further examination.
 
 """
-    else:
-        blurb += """As a larger _Filler VistA_, a large number of its patients only have filled IFCs - the patient's home VistA is elsewhere. 
-        
-As for users, most tend to only local consults or to a mix of local and filled consults.
-
-"""       
-
     mu += "{}\n\n{}\n\n".format(
         blurb,
         plotRef,
     )
     
     # ################################ Placer ######################
-    
-    HOMEVISNVISTAS = vistasOfVISNByOne(stationNo)     
-    
+            
+    # {"ANCHORAGE VA MEDICAL CENTER [363]", "BOISE VA MEDICAL CENTER [531]", "MANN-GRANDSTAFF VAMC [668]", "PORTLAND VA MEDICAL CENTER [648]", "ROSEBURG VA MEDICAL CENTER [653]", "SEATTLE VA MEDICAL CENTER [663]", "WHITE CITY VA MEDICAL CENTER [692]"}
+    VISN20s = {"363": "ANC", "531": "BOI", "687": "WWW", "668": "SPO", "648": "POR", "653": "ROS", "663": "PUG", "692": "WHI"}
     placerServicesFacilities = Counter()
     telerTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
     placerServicesTeleR = set()
     travelTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
     placerServicesTravel = set()
-    commonTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
-    placerServicesCommon = set()
-    spclzedTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
-    placerServicesSpclzed = set()
-    """
-    Enforced here as grouping done here -- TODO: move grouping of placers up to data
-    organization and not in mix of presentation.
-    
-    Of WRF's, only TELER has a local result (2 places for it - as a doc and in generic
-    associated_results). But it is a disappointing reference (in SPO -- TODO: check
-    more and wider by keeping all 8925 refs when typing) ala 
-    
-        >  "Please refer to Inter-facility Consult for results.\n\nA
-utomatically generated note - signature not required.\n \n  Electronically Filed
-: {DAY}\n                    by: {author-dictator}\n
- ",
-    """
-    def enforceOnlyTelerHasARnDoc(st):
-        if "associated results" in st:
-            raise Exception("WRF: TELER is only one expected to have local associated results")
-        if "tiu_result_narrative" in st:
-            raise Exception("WRF: TELER is only one expected to have local tiu - equivalent to associated resutls")
+    othrTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
+    placerServicesOthr = set()
     for i, service in enumerate(sorted(placerServices, key=lambda x: sum(st["_total"] for st in byServiceWRF[x]), reverse=True), 1):
         tttl = sum(st["_total"] for st in byServiceWRF[service])
         rf = singleValue(byServiceWRF[service][0], "routing_facility")
         placerServicesFacilities[rf] += tttl
         if service in telerAcServicesUsed:
-            groupLabel = "TELER"
             placerServicesTeleR.add(service)
             tblToUse = telerTBL
         elif re.search(r'(?i)travel', service):
-            groupLabel = "TRAVEL"
             placerServicesTravel.add(service)
             tblToUse = travelTBL
-        # follow-up is PUG, f/u is POR and these DISCHARGE FOLLOW UPs are only from big 
-        # (POR/PUG) to small and not vica-versa
-        elif re.search(r'(?i)(discharge f\/u|discharge follow\-up|virs |domiciliary|clinic f\/u|lodging)', service):
-            groupLabel = "COMMON"
-            placerServicesCommon.add(service)
-            tblToUse = commonTBL
         else: 
-            groupLabel = "SPECIALIZED"
-            placerServicesSpclzed.add(service)
-            tblToUse = spclzedTBL
+            placerServicesOthr.add(service)
+            tblToUse = othrTBL
         rfcntr = Counter()
         scntr = Counter()
         # Note: for Placer, only status varies (RF and by Defn, service is one)
         for st in byServiceWRF[service]:
-            if groupLabel != "TELER":
-                enforceOnlyTelerHasARnDoc(st)
             scntr[re.sub(r'100_01\-', '', singleValue(st, "cprs_status"))] += st["cprs_status"]["byValueCount"][singleValue(st, "cprs_status")]
-            if dad:
-                dad.dumpDocsOfST(groupLabel, st)
         serviceName = re.sub(r' +$', '', service.split(" [")[0]) 
         facMU = re.sub(r'4\-', '', rf)
-        facMU = f'__{facMU}__' if re.search(r'\-(\d+)\]', rf).group(1) in HOMEVISNVISTAS else facMU 
+        facMU = f'__{facMU}__' if re.search(r'\-(\d+)\]', rf).group(1) in VISN20s else facMU 
         tblToUse.addRow([
             # "__{}__".format(serviceName) if tttl/placerTTL > 0.1 else serviceName,
             serviceName,
@@ -505,9 +388,6 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
             muBVC(scntr)
         ])
         
-    if dad:
-        dad.finish()
-        
     mu += "## Placer\n\n"
     
     def summarizePlacerFacilities(services, groupLabel):
@@ -528,13 +408,13 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
         otherTTL = 0
         for facility in sorted(facilityCount, key=lambda x: facilityCount[x], reverse=True):
             facSNO = re.search(r'\-(\d+)\]', facility).group(1)
-            if facSNO in HOMEVISNVISTAS:
-                visnValues.append((facilityCount[facility], HOMEVISNVISTAS[facSNO]))
+            if facSNO in VISN20s:
+                visnValues.append((facilityCount[facility], VISN20s[facSNO]))
             else:
                 otherTTL += facilityCount[facility]
             facMU = re.sub(r'4\-', '', facility)
             # serviceNameMU = "__{}__".format(serviceName) if facilityCount[facility]/ssttl > 0.01 else serviceName
-            facMU = f'__{facMU}__' if facSNO in HOMEVISNVISTAS else facMU 
+            facMU = f'__{facMU}__' if facSNO in VISN20s else facMU 
             if cols[1] == ":Service":
                 row = [
                     facMU,
@@ -551,30 +431,26 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
         sankeyVIZData = {
             "title": f'{groupLabel} Placers', 
             "plotName": f'ifc{re.sub(" ", "", groupLabel)}', 
-            "plotMethod": "plotOutOtherSankey",
+            "plotMethod": "plotVISNSankey",
             "specs": {
-                "outValues": visnValues,
+                "toVISNValues": visnValues,
                 "toOtherTTL": otherTTL,
-                "fromLabel": "SOURCE" 
+                "fromLabel": "WWW" # TODO - make dynamic
             }
         }   
         plotData[f'placer{groupLabel}'] = sankeyVIZData    
         plotRef = muPlotRef(
-            sankeyVIZData,
+            sankeyVIZData["plotName"], 
+            sankeyVIZData["plotMethod"],
             imageDir
         )
         return tbl, set(facilityCount), ssttl, plotRef
         
     # Break into three.
     pbdmu = "" # placer breakdown mu
-    tttlTeleR = tttlTravel = tttlCommon = tttlSpclzed = 0
     
-    """
-    TODO: may put these down with specialized ... ie/ after travel and common ie/
-    part of specialized breakdowns and need to do others so IFCs covered in general.
-    """
-    if len(placerServicesTeleR):
-        ftbl, facilities, tttlTeleR, plotRef = summarizePlacerFacilities(placerServicesTeleR, "TeleReading")
+    ftbl, facilities, tttlTeleR, plotRef = summarizePlacerFacilities(placerServicesTeleR, "Placer TeleReading")
+    if tttlTeleR:
         pbdmu += "<span class='yellowIt'>{}</span> of the Placer Services are for __TeleReading__ to <span class='yellowIt'>{:,}</span> facilities ...\n\n".format(
             len(placerServicesTeleR), 
             len(facilities) 
@@ -584,100 +460,57 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
         pbdmu += telerTBL.md() + "\n\n"
 
     # For Travel, just use the facility table as 1-1 for most but PORT breaks that
-    # ... also note broken from "common" as not just within VISN!
-    # TODO: 
-    # - consider change to have toVISNs ... ie/ show all except lowest numbers
-    # including this VISN ... sunnier VISNs comment
-    # - ... also travel coincides with summer/holidays?
-    # - also who making these (docs or others?) 
-    if len(placerServicesTravel):
-        ftbl, facilities, tttlTravel, plotRef = summarizePlacerFacilities(placerServicesTravel, "Travel") 
-        pbdmu += "<span class='yellowIt'>{}</span> of the Placer Services denote __travel__ to remote facilities and represent <span class='yellowIt'>{}</span> of placed consults. These prove to be the exception to the rule that a VistA places most IFCs within its own VISN.\n\n".format(
-            len(placerServicesTravel), 
-            reportPercent(tttlTravel, placerTTL)
-        )
-        pbdmu += plotRef + "\n\n"
-        pbdmu += "By Facility ...\n\n"
-        pbdmu += ftbl.md() + "\n\n"
+    ftbl, facilities, tttlTravel, plotRef = summarizePlacerFacilities(placerServicesTravel, "Travel") 
+    pbdmu += "<span class='yellowIt'>{}</span> of the Placer Services denote __travel__ to remote facilities and represent <span class='yellowIt'>{}</span> of placed consults. These prove to be the exception to the rule that a VistA places most IFCs within its own VISN\n\n".format(
+        len(placerServicesTravel), 
+        reportPercent(tttlTravel, placerTTL)
+    )
+    pbdmu += plotRef + "\n\n"
+    pbdmu += "By Facility ...\n\n"
+    pbdmu += ftbl.md() + "\n\n"
 
-    """
-    A range of common "admin" (VIRS ...) and regular services (F/Us, lodging requests)
-    sent between systems of any size, whether they are fillers or placers.
-    
-    Discharge F/U ; VIRS ; Domiciliary ; Clinic F/U; LODGING ... -i
-    
-    Ex/ VIRS Coordinator - IFC Spokane [53], Spokane Clinic F/U - IFC [48], Discharge F/U - Spokane IFC [6]
-    
-    The fillers (see below) of Smaller VistAs are dominated by such requests. For larger
-    clinics, these common requests are proportionately less important.
-    """
-    if len(placerServicesCommon):
-        ftbl, facilities, tttlCommon, plotRef = summarizePlacerFacilities(placerServicesCommon, "Common") 
-        # All such consults stay within the VistA's VISN.
-        # ADD: if Placer VistA -- Note that as the filler section below shows, services like these dominate the list of consults filled by a Placer VistA such as this one.
-        pbdmu += """<span class='yellowIt'>{}</span> are \"Common\" Placer Services sent between both filler and placer VistAs. They represent a range of typical administrative (_\"VIRS\"_, _Lodging_, _Domiciliary_) and care requests to remote facilities (Follow ups). In this VistA, such requests represent <span class='yellowIt'>{}</span> of placed consults.
-  
-Note that equivalent services are filled by this VistA. Those are detailed in the _Filler Section_ below.      
-        
-""".format(
-            len(placerServicesCommon), 
-            reportPercent(tttlCommon, placerTTL)
-        )
-        pbdmu += plotRef + "\n\n"
-        pbdmu += "By Facility ...\n\n"
-        pbdmu += ftbl.md() + "\n\n"
-
-    # TODO: Portland etc -- very very few compared to WWW ie/ here fillers take the lot
-    # ... makes sense to group filler and placed specialized ... few if any for POR, all 
-    # Fill!
-    if len(placerServicesSpclzed):
-        ftbl, facilities, tttlSpclzed, plotRef = summarizePlacerFacilities(placerServicesSpclzed, "Specialized")        
-        # TODO: add later -- Placer VistA has many (relatively); a Filler has few - this is reversed in the Filler section    
-        pbdmu += "The balance, <span class='yellowIt'>{}</span>, of the Placer Services to <span class='yellowIt'>{:,}</span> facilities, are mainly for specialized clinical services and represent <span class='yellowIt'>{}</span> of placed consults. Once again, the vast majority go to other VISN 20 facilities but the flow is mainly from smaller facilities to larger with the small placing with the large.\n\n".format(
-            len(placerServicesSpclzed), 
-            len(facilities), 
-            reportPercent(tttlSpclzed, placerTTL)
-        )
-        pbdmu += plotRef + "\n\n"
-        pbdmu += "By Facility ...\n\n"
-        pbdmu += ftbl.md() + "\n\n"
+    ftbl, facilities, tttlOther, plotRef = summarizePlacerFacilities(placerServicesOthr, "Other")            
+    pbdmu += "The balance, <span class='yellowIt'>{}</span>, of the Placer Services to <span class='yellowIt'>{:,}</span> facilities, are for __other purposes__ and represent <span class='yellowIt'>{}</span> of placed consults. Once again, the vast majority go to other VISN 20 facilities.\n\n".format(
+        len(placerServicesOthr), 
+        len(facilities), 
+        reportPercent(tttlOther, placerTTL)
+    )
+    pbdmu += plotRef + "\n\n"
+    pbdmu += "By Facility ...\n\n"
+    pbdmu += ftbl.md() + "\n\n"
     
     """
     TODO PLOT: move to a horizontal once more categories with % and totals and overall #
         teler -----------xx
         travel -xx
-        common -------xx
-        spec -------------------xxx 
+        other -------------------xxx 
     """
     plotData["placerConsultsNServices"] = {
         "title": "Placed Consults and Services",
         "plotName": "ifcConsultsNServicesPlaced",
         "plotMethod": "plotCategoryBH",
         "rows": ["consults", "services"],
-        "columns": ["telereading", "travel", "common", "specialized"],
+        "columns": ["telereading", "travel", "other"],
         "data": [
             (
                 tttlTeleR,
                 tttlTravel,
-                tttlCommon,
-                tttlSpclzed
+                tttlOther
             ),
             (
                 len(placerServicesTeleR),
                 len(placerServicesTravel),
-                len(placerServicesCommon),
-                len(placerServicesSpclzed)
+                len(placerServicesOthr)
             ), # placed services 
         ]           
     }
     plotRef = muPlotRef(
-        plotData["placerConsultsNServices"],
+        plotData["placerConsultsNServices"]["plotName"], 
+        plotData["placerConsultsNServices"]["plotMethod"],
         imageDir
     )
     
-    # May break out 'VISN 20 CANCER CARE NAVIGATION TEAM IFC' -- another common, peer to
-    # peer service (all do placer and filler)
-    blurb = "There are <span class='yellowIt'>{:,}</span> __Placer Services__ - services that route/place consults __to__ <span class='yellowIt'>{:,}</span> other facilities. In VistA, a Placer Service identifies a specific remote (routing) facility which is always the main, default facility of a remote VistA. In this VistA, <span class='yellowIt'>{}</span> of the IFC Consults are \"Placed\" and the vast majority go __to other VISN 20 facilities__, facilities highlighted in bold in the tables below. Placer consults can be broken into four broad classes ...\n\n".format(
+    blurb = "There are <span class='yellowIt'>{:,}</span> __Placer Services__ - services that route/place consults __to__ <span class='yellowIt'>{:,}</span> other facilities. In VistA, a Placer Service identifies a specific remote (routing) facility which is always the main, default facility of a remote VistA. In this VistA, <span class='yellowIt'>{}</span> of the IFC Consults are \"Placed\" and the vast majority go __to other VISN 20 facilities__, facilities highlighted in bold in the tables below. Placer consults can be broken into three broad types ...\n\n".format(
         len(placerServices), 
         len(placerServicesFacilities), 
         reportPercent(placerTTL, wrfTTL)
@@ -689,13 +522,6 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
     mu += pbdmu # placer breakdown mu
     
     # ############################# Filler ###########################
-    
-    """
-    TODO: when break down
-    - Break into TeleR/Travel (out of VISN!)/Common/Specialized
-      ... or isn't TeleR just another form of specialized?
-    - consider NOT counting ANY discontinued except at the top => ie/ in summary => smaller tables
-    """
 
     tbl = MarkdownTable([":Filler Service", ":From Facilities", "\#", "Status'"])
     filledCntByPlacer = Counter()
@@ -710,10 +536,9 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
             rfcntr[placer] += cnt
             scntr[re.sub(r'100_01\-', '', singleValue(st, "cprs_status"))] += st["cprs_status"]["byValueCount"][singleValue(st, "cprs_status")]
         serviceName = re.sub(r' +$', '', service.split(" [")[0]) 
-        # TODO: from facilities don't appear to be ordered
         tbl.addRow([
             serviceName,
-            ", ".join(["{} [{:,}]".format(rf.split(" [")[0] if re.search(r'\[(\d+)\]', rf).group(1) not in HOMEVISNVISTAS else f'__{rf.split(" [")[0]}__', rfcntr[rf]) for rf in rfcntr]),
+            ", ".join(["{} [{:,}]".format(rf.split(" [")[0] if re.search(r'\[(\d+)\]', rf).group(1) not in VISN20s else f'__{rf.split(" [")[0]}__', rfcntr[rf]) for rf in rfcntr]),
             reportAbsAndPercent(tttl, fillerTTL),
             muBVC(scntr)
         ])
@@ -730,98 +555,63 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     print(f'Serializing Report to {userSiteDir}')
     open(f'{userSiteDir}ifcs.md', "w").write(mu) 
-    
+
+# ##################### Plot Creation #########################
+        
 """
-TMP HERE - move to IFCRawDataAnalysis.py for raw data analysis of IFC pieces
-... not part of .md flow per se
+Plot creation is separate --- may move into Visualize so common
 
-Want to see:
-- [ ] consult report_text (vs doc)
-  ... does filler get report_text off placer for its consult or its doc?
-- [I] documents on Placer and Filler side
-
-For Placer side first, dump documents of travel, telereading and common ... as
-traverse st's, can get docs dumped. Different groups possible ex/ Travel
-and TeleReading etc. and from Placer or Filler sides
-
-So Far:
-- for fraction of TELER with small number notes => will see a narrative ...
-
-        "report_text": "Please refer to Inter-facility Consult for results.\n\nA
-utomatically generated note - signature not required.\n \n  Electronically Filed
-: {DAY}\n                    by: {author-dictator}\n
- ",
-
-ie/ only TELER has document ref and then 
+ex/ plotDataFile = "plotDataIFC.json" ie/ plotDataFileSuffix="IFC"
 
 TODO:
-- "reason_for_request" ... does this have initial VIRS etc text? [same remotely?] ie/ just a "text copy"?
-"""
-import shelve
-from fmqlutils.cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator
-class DumpAndAnalyzeData:
-    def __init__(self, stationNo, cutDate):
-        self.__docsByGroupLabel = defaultdict(list)
-        
-        cutDateDT = datetime.strptime(cutDate.split("T")[0], "%Y-%m-%d")
-        yrBeforeDateDT = cutDateDT - relativedelta(years=1)
-        yrBeforeDate = datetime.strftime(yrBeforeDateDT, "%Y-%m-%d")
-        onAndAfterDay = yrBeforeDate
-        print(f'Calculated year before date is {yrBeforeDate}')
-        
-        tmpWorkingDir = f'{VISTA_DATA_BASE_DIR}{stationNo}/TmpWorking/'
-        all8925ByIENDBFL = f'{tmpWorkingDir}all8925ByIEN{onAndAfterDay}.db'
-        if os.path.isfile(all8925ByIENDBFL):
-            print(f'8925 Types DB reduction in {all8925ByIENDBFL} done already')
-            self.__8925ByIEN = shelve.open(all8925ByIENDBFL, flag='r')
-            return
-        dataLocn = f'{VISTA_DATA_BASE_DIR}{stationNo}/Data/'
-        if not os.path.isfile(dataLocn + "8925-0.zip"):
-            raise Exception("8925 not available for reduction")
-        print(f'No 8925 Reduction for {onAndAfterDay} on so making')
-        store = FMQLReplyStore(dataLocn)
-        startAtReply = store.firstReplyFileOnOrAfterCreateDay("8925", "entry_date_time", onAndAfterDay)
-        if startAtReply == "":
-            raise Exception("Can't find a start at reply for {}".format("8925"))
-        print("Starting with reply {}".format(startAtReply))
-        resourceIter = FilteredResultIterator(dataLocn, "8925", startAtReply=startAtReply)
-        all8925ByIEN = shelve.open(all8925ByIENDBFL)
-        for i, resource in enumerate(resourceIter, 1):
-            if i % 100000 == 0:
-                print("Another 100K to {:,}".format(i))
-            all8925ByIEN[str(resource["_id"].split("-")[1])] = resource
-        all8925ByIEN.close()    
-        self.__8925ByIEN = shelve.open(all8925ByIENDBFL, flag='r')
+- ifcPlTravelVISNSankey (from Travel PlacersVISNSankey) -- .md change too
+- all consult services categ ... -> ifcAllConsultsCategory 
+- remove 0 ie/ NO OTHER!
+- map SEA -> PUG, MAN -> SPO
 
-    """
-    Ex/ dumpDocsOfST('placerTeleR', st)
-    
-    Only TELER has tiu_result_narrative (and associated_results)
-    See limited # of TELER where doc has 'see elsewhere' text but where?
-    - associated_results <=> tiu_result_narritive
-    - remote_results == VPTR for 8925's remotely [ie/ doc not transferred]
-    
-    No Document Locally: VIRS, Travel etc - all have remote doc refs
-    
-    OTHER:
-    - note the "from" 44's in sts ... need these for all service types or see it in patient_location
-    """
-    def dumpDocsOfST(self, groupLabel, st):
-        if "tiu_result_narrative" not in st:
-            return
-        if "associated_results" not in st:
-            raise Exception("Expected 'associated_results' if 'tiu_result_narrative'")
-        if "byValueCount" not in st["tiu_result_narrative"]:
-            return
-        for ref in st["tiu_result_narrative"]["byValueCount"]:
-            tiuIEN = str(re.search(r'\[8925-(\d+)\]', ref).group(1))
-            self.__docsByGroupLabel[groupLabel].append(self.__8925ByIEN[tiuIEN])
-            # print(json.dumps(st, indent=4))
+"""
+
+def makePlots(stationNo, plotDataFileSuffix):   
+
+    from visualize import Visualize
+    viz = Visualize("Images")
+    plotData =  json.load(open(f'{VISTA_DATA_BASE_DIR}{stationNo}/TmpWorking/plotData{plotDataFileSuffix}.json'))
+    for plotDataName in plotData:
+        plotDataInfo = plotData[plotDataName]
+        print(f'Remaking {plotDataInfo["title"]}')
+        method = getattr(viz, plotDataInfo["plotMethod"])
+        if "rows" in plotDataInfo:
+            arg = viz.makeDF(
+                data=plotDataInfo["data"],
+                columns=plotDataInfo["columns"],
+                rows=plotDataInfo["rows"]
+            )
+        elif "specs" in plotDataInfo:
+            arg = plotDataInfo["specs"]
+        else:
+            raise Exception("Expect plot data for DF or as 'specs'")
+        plotRef = method(arg, plotDataInfo["title"], plotDataInfo["plotName"]) 
+        print(f'\texpect reference to {plotRef} in report')
         
-    def finish(self):
-        for groupLabel in self.__docsByGroupLabel:
-            json.dump(self.__docsByGroupLabel[groupLabel], open(f'{groupLabel}.json', "w"), indent=4)
-            print(f'Look in {groupLabel}.json for documents of {groupLabel}')
+def muPlotRef(plotName, plotMethodName, flushToDirectory):
+    """
+    # Want separate from visualize for now BUT possible break
+    # as separate from Visualize naming impl
+    
+    ex/ ![ifcTravelVISNSankey](Images/ifcTravelVISNSankeyBGW.svg)
+    """
+    def svgPlotFile(plotName, plotMethodName, flushToDirectory):
+        """
+        So don't have to make the plot but can know its name as making md
+        Compatible with Visualize.__flushPlot's naming of SVG's
+        """
+        coreMethodName = re.sub("plot", "", plotMethodName)
+        coreMethodNameU = coreMethodName[0].upper() + coreMethodName[1:]
+        plotFilePrefix = f'{plotName}{coreMethodNameU}'
+        plotFile = f'{flushToDirectory}/{plotFilePrefix}BGW.svg'
+        return plotFile, plotFilePrefix
+    plotFile, plotFilePrefix = svgPlotFile(plotName, plotMethodName, flushToDirectory)
+    return "![{}]({})".format(plotFilePrefix, plotFile)
     
 """
 > https://hackingandslacking.com/using-hierarchical-indexes-with-pandas-38057cf1d7e
