@@ -9,16 +9,26 @@ from collections import defaultdict, Counter
 from datetime import datetime, date, timedelta
 from dateutil.relativedelta import relativedelta
 
-from fmqlutils.cacher.cacherUtils import TMPWORKING_LOCN_TEMPL, FMQLReplyStore, FilteredResultIterator, DATA_LOCN_TEMPL, metaOfVistA
+from fmqlutils import VISTA_DATA_BASE_DIR
+from fmqlutils.cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator, metaOfVistA
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent, muBVC
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVCOfSTProp
 
-from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200, flattenPropValues
+from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200, reduce4, flattenPropValues
 
 """
 TODO: ac finish
 - 4 way service including enforce for routing
   ... and need reduce4 so using stationNo
+  
+REDO ACQ ACTIVITY AGAIN TO:
+- break by READING VistA ie/ same rep but per reading VistA!
+- graphics of activity ... see if redo WWW ... per week/day/month? and per reader?
+  ... over the year / per activity type (ie/ per service?)
+  ... another go ala s... diagram
+------
+- idea (if available) of patient reg delay
+THEN DO READER SIDE AND THEIR CONSULTs ... needs breakdown ---- MAY BE FRIDAY
 """
 
 """
@@ -193,7 +203,7 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         ) if "ifc_role" in type2006_5849YR1E else 0,
     """
     consultIFCCount = type2006_5849YR1E["is_consult_ifc"]["count"] if "is_consult_ifc" in type2006_5849YR1E else 0
-    mu += """An _Acquisition site_ imports images and submits them for telereading \"consults\". These are tracked in _TELEREADER READ/UNREAD LIST (2006.5849)_. In this site. There are <span class='countHigh'>{:,}</span> total consults between {} and {}. Of these, <span class='countHigh'>{}</span> are in this LIST, <span class='countHigh'>{}</span> of these are interfacility consults (IFCs), where another facility performs the consult while the balance, <span class='countHigh'>{}</span>, are performed locally.
+    mu += """An _Acquisition site_ imports images and submits them for telereading \"consults\". These are tracked in _TELEREADER READ/UNREAD LIST (2006.5849)_. In this site, there are <span class='countHigh'>{:,}</span> total consults between {} and {}. Of these, <span class='countHigh'>{}</span> are in this LIST, <span class='countHigh'>{}</span> of these are interfacility consults (IFCs), where another facility performs the consult while the balance, <span class='countHigh'>{}</span>, are performed locally.
     
 """.format(
         type123YR1["_total"],
@@ -270,6 +280,9 @@ def webReportAcquisition(stationNo, onAndAfterDay, upToDay):
         if status not in st2006_5849ByStatus:
             continue
         for st in sorted(st2006_5849ByStatus[status], key=lambda x: x["_total"], reverse=True):
+            # POR has this for DELETED - TODO, examine
+            if "consult_audit_trail_transitions" not in st:
+                continue
             for trans in sorted(st["consult_audit_trail_transitions"]["byValueCount"], key=lambda x: st["consult_audit_trail_transitions"]["byValueCount"][x], reverse=True):
                 if status != currentStatus:
                     currentStatus = status
@@ -394,16 +407,104 @@ The client __[Telereader Configurator](https://www.va.gov/vdl/documents/Clinical
     return mu
 
 """
-TODO: 
-- FOCUS ON MERGING TABLES so see AC Through Routing from two and not one file ... ALSO service == Instit + Specialty
-- MORE FOR READING ie/ how do reading services (ie/ stuff in from ...) work
+Acquisition Service is effectively a combination of information from 123.5 and 2006_5841
 
-Note: Specialty 2006_84 (DERMATOLOGY, EYE CARE, ...) have no extra nuances except an acronym (all are active, all are CLIN) so NOT detailing in its own table
+TODO:
+- 
+- tie in tiu, creation trigger (once I'm done with it)
+- consider a synthesized name of SPEC PROC ABBREVS (SNO AC/SNO READ VISTA)
 """
 def webReportConfigurationAcquisition(stationNo):
 
-    dataLocn = DATA_LOCN_TEMPL.format(stationNo)      
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
+    ifcServiceIndex = dict((cs["_id"], cs) for cs in FilteredResultIterator(dataLocn, "123_5") if "ifc_routing_site" in cs)
+    institByIEN = reduce4(stationNo)
+    snoByNationalInstitId = dict(("4-" + ien, institByIEN[ien]["station_number"]) for ien in institByIEN if "isNational" in institByIEN[ien] and "station_number" in institByIEN[ien])
+    
+    asResourceIter = FilteredResultIterator(dataLocn, "2006_5841")
+    serviceCount = 0
+    byRoutingSite = defaultdict(list)
+    locals = []
+    for resource in asResourceIter:
+        serviceCount += 1
+        row = [
+            "__{}__".format(resource["name"]["label"]),
+            "{} [{}]".format( # 2006.5841
+                resource["acquisition_site"]["label"],
+                snoByNationalInstitId[resource["acquisition_site"]["id"]]
+            ),
+            "{} [{}]".format( # 2006.5841
+                resource["specialty_index"]["label"],
+                resource["specialty_index"]["id"].split("-")[1]
+            ),
+            "{} [{}]".format( # 2006.5841
+                resource["procedure_index"]["label"],
+                resource["procedure_index"]["id"].split("-")[1]
+            )
+        ]
+        # If not just LOCAL
+        if resource["name"]["id"] in ifcServiceIndex: # 123.5
+            ifcService = ifcServiceIndex[resource["name"]["id"]]
+            row.append("__{}__".format(ifcService["ifc_remote_name"]))
+            routingSite = ifcService["ifc_routing_site"]
+            remoteSiteRef = "{} [{}]".format(
+                routingSite["label"],
+                snoByNationalInstitId[routingSite["id"]]
+            )
+            byRoutingSite[remoteSiteRef].append(row)
+            print(ifcService["associated_stop_code"])
+        else:
+            locals.append(row)
+        trigger = resource["unread_list_creation_trigger"]
+        tiuNoteFile = resource["tiu_note_file"] if "tiu_note_file" in resource else ""
+        # associatedStopCode = "associated_stop_code
+    # ADD NOTE:
+    # MAND ... ADD STOP CODES as they are using em!!!!!
+    mu = "### Acquisition Role\n\n"
+    mu += "There are <span class='countHigh'>{:,}</span> consult services for Telereading by <span class='countHigh'>{:,}</span> remote sites.{}\n\n".format(
+        serviceCount,
+        len(byRoutingSite),
+        " In addition, <span class='countHigh'>{:,}</span> services are read locally.".format(len(locals)) if len(locals) else ""
+    )
+    for rs in sorted(byRoutingSite):
+        # "All map to one service in Boise
+        acSites = set(row[1] for row in byRoutingSite[rs])
+        remoteServices = set(row[-1] for row in byRoutingSite[rs])
+        cols = [":Service", ":Acquisition Site", ":Specialty", ":Procedure"]
+        if len(remoteServices) > 1:
+            cols.append(":Remote Service")
+        tbl = MarkdownTable(cols)
+        for row in sorted(byRoutingSite[rs], key=lambda x: x[1]): # group by ac site
+            if len(remoteServices) == 1:
+                row.pop()
+            tbl.addRow(row)
+        if len(acSites) > 1:
+            mu += "__{}__ receives <span class='countHigh'>{:,}</span> types of consult service from <span class='countHigh'>{:,}</span> acquisition sites{} ...\n\n".format(
+                rs, 
+                len(byRoutingSite[rs]), 
+                len(acSites),
+                ", all sent to one remote service __{}__".format(list(remoteServices)[0]) if len(remoteServices) == 1 else "" 
+            )
+        else:
+            mu += "__{}__ has <span class='countHigh'>{:,}</span> services ...\n\n".format(rs, len(byRoutingSite[rs]))
+        mu += tbl.md() + "\n\n"
+    if len(locals):
+        tbl = MarkdownTable([
+            ":Service",
+            ":Acquisition Site",
+            ":Specialty",
+            ":Procedure"
+        ])
+        mu += "The <span class='countHigh'>{:,}</span> locally read services are ...\n\n".format(len(locals))
+        for row in sorted(locals, key=lambda x: x[0]):
+            tbl.addRow(row)
+        mu += tbl.md() + "\n\n"
+    
+    return mu
 
+def webReportConfigurationAcquisitionOld(stationNo):
+
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
     _123_5sOf2006_5841s = set()    
     resourceIter = FilteredResultIterator(dataLocn, "2006_5841")
     bySpecialtyIndex = defaultdict(list)
@@ -527,9 +628,9 @@ def webReportConfigurationReader(stationNo):
 """.format(i)
         fmu += tbl.md() + "\n\n"
         return fmu
-    
-    dataLocn = DATA_LOCN_TEMPL.format(stationNo)
-    
+        
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
+
     mu = "### Reader Role\n\n"
     mu += mu2006_5842(dataLocn)
     mu += mu2006_5843(dataLocn)
@@ -617,7 +718,7 @@ Note: this is the direction from Consult to Queue. This COULD become type based
 """
 def isolateMissingConsults(stationNo): # TMP - move to use typer in mu
 
-    csJSNFl = TMPWORKING_LOCN_TEMPL.format(stationNo) + "telereaderQueueConsultSummary.json"
+    csJSNFl = "{}{}/TmpWorking/".format(VISTA_DATA_BASE_DIR, stationNo) + "telereaderQueueConsultSummary.json"
     try:
         trqConsultSummary = json.load(open(csJSNFl))
     except:
@@ -680,7 +781,7 @@ def isolateMissingConsults(stationNo): # TMP - move to use typer in mu
 """
 def reduce123_5ForIFC(stationNo):
     
-    jsnFl = TMPWORKING_LOCN_TEMPL.format(stationNo) + "ifcServiceByIEN.json"
+    jsnFl = "{}{}/TmpWorking/".format(VISTA_DATA_BASE_DIR, stationNo) + "ifcServiceByIEN.json"
     try:
         ifcServiceByIEN = json.load(open(jsnFl))
     except:
@@ -688,7 +789,7 @@ def reduce123_5ForIFC(stationNo):
     else:
         return ifcServiceByIEN
     
-    dataLocn = DATA_LOCN_TEMPL.format(stationNo)
+    dataLocn = "{}{}/Data/".format(VISTA_DATA_BASE_DIR, stationNo)   
     resourceIter = FilteredResultIterator(dataLocn, "123_5")
     ifcServiceByIEN = {}
     for i, resource in enumerate(resourceIter, 1):
