@@ -5,11 +5,8 @@ import sys
 import json
 import re
 from collections import defaultdict, Counter
-from datetime import datetime
-from dateutil.relativedelta import relativedelta
 
 import matplotlib.pyplot as plt
-import matplotlib.ticker as tick
 import matplotlib.dates as mdates
 from matplotlib.sankey import Sankey
 import pandas as pd
@@ -21,16 +18,8 @@ except:
     
 """
 TODO NOW:
-- simple overview of when bar, boxplot, scatter + shows seaborn (simple) of all
-  - https://towardsdatascience.com/a-beginners-guide-to-plotting-your-data-python-r-11e435262ac1 [review to tighten own]
-- style: https://medium.com/analytics-vidhya/drastically-beautifying-visualizations-with-one-line-styling-plots-35a5712c4f54 <------ shows built in plt styles (not just seaborn) and can invoke easily. [FRIDAY] ex/ 538 or etc
-  - see how these style files created (plt.style.use = https://matplotlib.org/3.3.0/tutorials/introductory/customizing.html) ... take one and then work own over months
-- and review overall: http://www.storytellingwithdata.com/
-
-Playfair's bars
-- its orientation (vertical and horizontal bar charts) [H or V]
-- and its arrangement (e.g. grouped and stacked bar charts) [G or S] 
-ie/ denote types ... and variations of all these bars
+- (tracked in IFC -- move it over to the two phase and these images forms including sankeys + QA with SPO)
+- move webReportDirectoriesLess over too [will need refactor to be properly automatic and driven by config?]
 """
 
 """    
@@ -60,8 +49,7 @@ TODO:
 
 Approach to code: this is THE GENERIC VERSION. Reports may have custom plotrs too in a separate module - ex/ plotUsers.py etc.
 
-BACKGROUND (on the "art"):
-===========================
+Context: 
 
 > We have Playfair to thank for many of the popular graphs that we use today, including the bar graph (why pies bad: http://www.perceptualedge.com/articles/08-21-07.pdf) + get posters.
    - https://www.amazon.com/Playfairs-Commercial-Political-Statistical-Breviary/dp/0521855543
@@ -71,31 +59,20 @@ but
 
 > Edward Tufte once said that “the only worse design than a pie chart is several of them, for then the viewer is asked to compare quantities located in spatial disarray both within and between pies” ... see ([save pies for the desert](http://www.perceptualedge.com/articles/08-21-07.pdf))
     
-This [article](https://medium.com/nightingale/3rd-wave-data-visualization-824c5dc84967) goes into the first (Tufte) and Second (D3 et al) waves with references to other essays on forms and approaches. TODO <------- follow it around and people like:
-    - https://twitter.com/jburnmurdoch
-    
 Visuals to Add:
-================
 - Radial (for time series) ... by year ... relative additions ... https://medium.com/nightingale/from-the-battlefield-to-basketball-a-data-visualization-journey-with-florence-nightingale-c39571686dfc
   and https://www.pythonprogramming.in/plot-polar-graph-in-matplotlib.html [polar]
   ... can see max ever and then each year within it and then each system in that
 - TreeMap (squares size wanted for Workload per site and then relative
 size of each square for each clinic/entity?)
   - import plotly.graph_objects as go / fig = go.Figure(go.Treemap
-  
-  
-Not much magic here. We import lru_cache and decorate a function that generates Fibonacci numbers with it.
-
-    @lru_cache(maxsize=None)
-    def do(x):  
-
 """
 class Visualize:
 
     def __init__(self, flushToDirectory=""): 
         self.__flushToDirectory = flushToDirectory or "Images"
         
-    def svgPlotFile(self, plotName, plotMethodName): # rem: kargs inside plotName
+    def svgPlotFile(self, plotName, plotMethodName):
         """
         So don't have to make the plot but can know its name as making md
         Compatible with __flushPlot's naming of SVG's
@@ -114,12 +91,12 @@ class Visualize:
         
         Ex/ 
         
-            data = [ (aka rows)
+            data = [
                 ("val col1", "val col2", "val col3") # of row 1
                 ("val col1", "val col2", "val col3") # of row 2
             ]
             columns: ["col1 name", "col2 name", "col3 name"],
-            rows: ["row1 name", "row2 name"] (aka rowLabels)
+            rows: ["row 1", "row 2"]
         
         """
         df = pd.DataFrame(
@@ -134,40 +111,39 @@ class Visualize:
         plotFilePrefix = f'{plotName}IsHasBH' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
                 
-    def plotCategoryBH(self, df, title, plotName):
-        plotCategoryBH(df, title)
+    def plotCategoryBH(self, df, title, plotName, usePerc=True):
+        plotCategoryBH(df, title, usePerc)
         plotFilePrefix = f'{plotName}CategoryBH' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
         
-    def plotOutOtherSankey(self, specs, title, plotName):
-        plotOutOtherSankey(specs["outValues"], specs["toOtherTTL"], specs["fromLabel"], title)
-        plotFilePrefix = f'{plotName}OutOtherSankey' # add gr type to prefix        
+    def plotVISNSankey(self, specs, title, plotName):
+        plotVISNSankey(specs["toVISNValues"], specs["toOtherTTL"], specs["fromLabel"], title)
+        plotFilePrefix = f'{plotName}VISNSankey' # add gr type to prefix        
         return self.__flushPlots(plotFilePrefix)
+                
+    # TODO: there are two! <----------- ******
+    def plotCommons(self, commonTotals, dirName):
+        # imgRefPercs = flushPlots("{}CommonPercents".format(dirName), flushToDirectory)
+        return plotCommons(commonTotals, dirName)
         
-    # Two variations - absolute and usePerc
-    def plotCategoryBSV(self, totalsByCatagByStackId, title, plotName, usePerc=False):
-        plotCategoryBSV(totalsByCatagByStackId, title, usePerc)
-        plotFilePrefix = f'{plotName}CategoryBSV'    
-        return self.__flushPlots(plotFilePrefix)
-                        
-    def plotDailies(self, dailies, title, plotName, colNameById, start='2020-01', end='2020-02'):
-        plotDailies(dailies, title, colNameById, start, end)
-        plotFilePrefix = f'{plotName}Dailies{start}-{end}' # add gr type to prefix
+    def plotDailies(self, dailies, dirName, vistaNamesPlus, start='2020-01', end='2020-02'):
+        plotDailies(dailies, dirName, vistaNamesPlus, start, end)
+        plotFilePrefix = f'{dirName}Additions{start}-{end}' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plot365RollingMean(self, dailies, title, plotName, colNameById, start="1987-01", end=""):
-        plot365RollingMean(dailies, title, colNameById, start, end)
-        plotFilePrefix = f'{plotName}365RollingMean{start}-{end}' # add gr type to prefix
+    def plot365RollingMean(self, dailies, dirName, vistaNamesPlus):
+        plot365RollingMean(dailies, dirName, vistaNamesPlus)
+        plotFilePrefix = f'{dirName}RollingMean365' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plotMonthlyResamples(self, dailies, title, plotName, colNameById, start='2018-01', end='2020-02'):
-        plotMonthlyResamples(dailies, title, colNameById, start, end)
-        plotFilePrefix = f'{plotName}MonthlyResamples{start}-{end}' # add gr type to prefix
+    def plotMonthlyResamples(self, dailies, dirName, vistaNamesPlus):
+        plotMonthlyResamples(dailies, dirName, vistaNamesPlus)
+        plotFilePrefix = f'{dirName}ResampleM18-20' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
-    def plotTotalsPerYear(self, dailies, title, plotName, colNameById, start="2015", end="2020", snosOnly=True):
-        plotTotalsPerYear(dailies, title, colNameById, start, end, snosOnly)
-        plotFilePrefix = f'{plotName}TotalsPerYear{start}-{end}' # add gr type to prefix
+    def plotTotalsPerYear(self, dailies, dirName, vistaNamesPlus):
+        plotTotalsPerYear(dailies, dirName, vistaNamesPlus)
+        plotFilePrefix = f'{dirName}Totals15-20' # add gr type to prefix
         return self.__flushPlots(plotFilePrefix)
 
     def __flushPlots(self, plotFilePrefix):
@@ -220,30 +196,21 @@ def plotIsHasBH(countsNTotal, usePerc=True):
     - [2] side by side comparison of percent of one attribute (ex/ Male)
     for many entities (ex/ VistA 1, VistA 2 ...). Entities are the Y Axis and the 
     LVALUE of the dictionary passed in.
-    but not for category/enum information (ex/ Male vs Female, ALL vs Some VistAs)
     
     Options: usePerc - use percentages vs usePerc = False => absolutes shown
-        
+    
+    Not For: category/enum information (ex/ Male-Female, ALL | 1 | 2 | 3 VistAs etc)
+    
         {
-            "__entityName": ...
+            "__name": ...
             "__total": #,
-            "assertion 1": # <= total, (ex/ in one vista, sent to other sites ...)
+            "Y1": # <= total,
             ...
         }
         
     The Percents are calculated inside this function. The #'s passed are absolutes. The
     attributes are reordered by size.
     
-    Note: alternative often used is a DL with form
-    
-        blah blah ..
-                  <---- newline
-            assertion 1
-             : %  <---- one space, colon
-                  <---- newline til next entry
-            assertion 2
-             : %
-    
     TODO (cosmetic)
     - see if can do a DF perc (see tip at bottom of this: 
     https://python-graph-gallery.com/13-percent-stacked-barplot/ with axis etc)
@@ -253,9 +220,9 @@ def plotIsHasBH(countsNTotal, usePerc=True):
       - white BG
       - no horiz lines (got this); verticals light grey not white so show
     """
-    if not ("__entityName" in countsNTotal and "__total" in countsNTotal):
-        raise Exception("'countsNTotal' needs __entityName and __total")
-    name = countsNTotal["__entityName"]
+    if not ("__name" in countsNTotal and "__total" in countsNTotal):
+        raise Exception("'countsNTotal' needs __name and __total")
+    name = countsNTotal["__name"]
     total = countsNTotal["__total"]
     index = []
     data = {"SET": [], "NOTSET": []}
@@ -273,13 +240,12 @@ def plotIsHasBH(countsNTotal, usePerc=True):
         data["NOTSET"].append(notSetValue)
 
     sns.set() 
-    fig, ax = plt.subplots(figsize=(11, 4)) # fixed 11/4
-    ax.set_title("{:,} {}".format(total, name.capitalize()))
+    fig, ax = plt.subplots(figsize=(11, 4))
+    ax.set_title("{:,} {}s".format(total, name.capitalize()))
     width = .75 # the width of the bars: can also be len(x) sequence
     ax.barh(index, data["SET"], width, edgecolor="none")
     ax.barh(index, data["NOTSET"], width, left=data["SET"], color="lightgray", edgecolor="none")
     ax.xaxis.set_major_formatter(plt.NullFormatter())
-    # ax.xaxis.set_major_formatter(tick.FuncFormatter(reformat_large_tick_values));
     ax.margins(0) # this gets rid of the right spine's extra margin (ax.spines["right"])
     ax.yaxis.set_tick_params(pad=15) # note: couldn't bold em easily ... LaTeX TODO
     ax.tick_params(axis='both', which='both',length=0)
@@ -295,31 +261,24 @@ def plotIsHasBH(countsNTotal, usePerc=True):
                 xpos = x + total/100
             else:
                 xpos = x - total/12
-            valToShow = "{:,}".format(reformat_large_tick_values(x))
+            valToShow = "{:,}".format(x)
         plt.text(xpos, i, valToShow, fontsize=11, fontweight="bold", va='center')  
     plt.tight_layout()
     
-def plotCategoryBH(df, title):
+def plotCategoryBH(df, title, usePerc=True):
     """
-    Category (vs isHas) Bar Horizontal for different items with the same 
-    category breakdowns. Alternative to pie charts.
-    
-                       title (tip: put a # message in the title)
-    
-    rowLabel1    |    %col2         | %col2     | %col3 | #
-    
-    rowLabel2    |
-   
-    where key will be col1, col2, col3 and the rows have data with the same breakdown (col1, ...) but can be different orders ie/ total up differently. Ex/ records and files.
-           
+    Category (vs isHas) Bar Horizontal for:
+    - different items with the same category breakdowns 
+    ... do more to refine (could do all same totals => show total?)
+        
     Accepts df. Ex/
-        rowLabels = ["rowLabel1", "rowLabel2", "rowLabel3", "rowLabel4"]
-        cols = ["col1", "col2", "col3"]
-        rows = [(89361, 5907, 0), (262, 209, 0), (390, 336, 44), (13151, 3454, 500)]    
+        rows = ["consults", "services", "users", "patients"]
+        columns = ["non ifc", "ifc", "both"]
+        data = [(89361, 5907, 0), (262, 209, 0), (390, 336, 44), (13151, 3454, 500)]    
         df = pd.DataFrame(
-            rows,
-            columns=cols,
-            index=rowLabels
+            data,
+            columns=columns,
+            index=rows
         )
     Will turn into percentages.
     (Tip: df = df.drop("services"))
@@ -329,13 +288,11 @@ def plotCategoryBH(df, title):
     - consider % on right if #'s don't all total to the same ie/ ala my % of % tables
     as this is a replacement and also order down on those? ie/ lessor total % in lower
     rows
-    
-    REF:    https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/horizontal_barchart_distribution.html#sphx-glr-gallery-lines-bars-and-markers-horizontal-barchart-distribution-py <--- NOT this method but the look side to side is ok
+    https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/horizontal_barchart_distribution.html#sphx-glr-gallery-lines-bars-and-markers-horizontal-barchart-distribution-py <--- NOT this method but the look side to side is ok
     
     """        
-
-    # perc version used for all but end total!
-    dfp = df.div(df.sum(axis=1), axis=0).mul(100).round(3) 
+    # df = df.drop("services")
+    dfp = df.div(df.sum(axis=1), axis=0).mul(100).round(3)
         
     sns.set() 
     figsize = (9, 2 * len(df.index))
@@ -358,18 +315,6 @@ def plotCategoryBH(df, title):
     Alt would be put index (first col) into rows at pos 0 and always give its
     values and then values of next col's index in all rows but better to keep
     columns (index) separate
-    
-    or TODO: iterrows (as just in => cols only)
-    
-        con = []
-        for i, row in df.iterrows():
-            g = {‘Customer ID #: ‘+str(row[‘customer_id’]): 
-              {‘Address’: row[‘address’],
-               ‘Post Code’: row[‘postcode’],
-               ‘State’: row[‘state’], 
-               ‘Country’: row[‘country’]
-            }}
-            con.append(g)
     """    
     for i, colName in enumerate(dfp.columns):
         # TODO: want first on top -- must fix
@@ -382,56 +327,19 @@ def plotCategoryBH(df, title):
         ttlsSoFar = [a + b for a, b in zip(ttlsSoFar, dfp[colName].values)]
     ax.legend(fontsize=10, loc='upper left')
     
-    rowValSoFarByRowIdx = Counter()
-    for colName in df.columns: # go across
-        for i, val in enumerate(dfp[colName]): # go down
-            rowValSoFarByRowIdx[i] += val
-            if val < 3:
-                continue
-            display = f'{round(val)}%'
-            xoffset = round(len(display)/1.25)
-            xForVAL = rowValSoFarByRowIdx[i] - (round(val/2) + xoffset)
-            plt.text(xForVAL, i, display, fontsize=11, fontweight='bold', va='center')
+    # for j, colName in enumerate(df.columns):
+    j = 0 # TODO: want all except small text %'s
+    for i, val in enumerate(dfp[df.columns[j]]): # first part of each only
+        if val < 30:
+            continue # don't put in label if smaller than 30%
+        plt.text(val-round(figsize[0] * 0.7), i, f'{round(val)}%', fontsize=11, fontweight='bold', va='center')
     ttls = df.sum(axis=1)
     for i, ttl in enumerate(ttls):
-        ttlToShow = reformat_large_tick_values(ttl)
-        x = 100 + round(figsize[0] * 0.25)
-        plt.text(x, i, ttlToShow, fontsize=11, fontweight='bold', va='center')
+        plt.text(100 + round(figsize[0] * 0.3), i, ttl, fontsize=11, fontweight='bold', va='center')
     plt.tight_layout()
     
 """
-Classic Vertical Stackbar (BSV) for a category breakdown, total or same size percentages. 
-
-TODO MORE WHEN TIME: 
-> plt.rcParams.update({'font.size': 22})
-and
-params = {'legend.fontsize': 'large',
-          'figure.figsize': (20,8),
-          'axes.labelsize': size,
-          'axes.titlesize': size,
-          'xtick.labelsize': size*0.75,
-          'ytick.labelsize': size*0.75,
-          'axes.titlepad': 25}
-plt.rcParams.update(params)
-ie/ calculate and set all
-
-TMP FIRST: just totals ...
-
-        totalsByCatagByStackId = {
-            "WWW": {
-                NONE: 7499,
-                2: 7802,
-                3: 27625,
-                ALL: 177852
-            },
-            "SPO": {
-                NONE: 20478,
-                2: 33649,
-                3: 92134,
-                ALL: 177852
-            },
-            ...
-        }
+Classic Stackbar for a category breakdown and then equivalent for percentages. Vertical.
 
 TODO:
 - GENERALIZE (ex/ https://medium.com/swlh/converting-nested-json-structures-to-pandas-dataframes-e8106c59976e + https://python-graph-gallery.com/13-percent-stacked-barplot/ comments)
@@ -453,103 +361,76 @@ Key TODO for this and the totals above is to move to DataFrame proper
 ie/ want DF from data, index and then [1] percentage from base one (don't 
 manually calc) [2] as below for TimeSeries, drive off DF
 """
-def plotCategoryBSV(totalsByCatagByStackId, title, usePerc=False): # bar, stacked, vertical
-
+def plotCommons(commonTotals, dirName):
     """
     matplotlib direct: https://matplotlib.org/api/_as_gen/matplotlib.axes.Axes.bar.html#matplotlib.axes.Axes.bar
     pandas.dataframes.plot (kind="barh")
     """
         
     """
-    Turn StackId into index and catag "owns" dictionary ie/
-        www: {"ALL": # ---> {"ALL: [# for WWW, ...
-    which lets us paint the graph layer by layer from bottom to top
-    
     X axis (index) of VistA (SNO) (vs date in time series)
     Y axis as int amounts (same as time series)
     Legend/color/column shown to be 1, 2, 3, 4 (ala VistA (SNO) in time series)
     """
     # want in order of size, smallest to the right
     index = []
-    data = defaultdict(list) # index is legend/Y ie/ ALL, 3, ...; X is list index
-    masterCatagOrder = [] # ensuring stacks have same order
-    for i, stackId in enumerate(sorted(totalsByCatagByStackId, key=lambda x: sum(totalsByCatagByStackId[x][k] for k in totalsByCatagByStackId[x]))):
-        index.append(stackId)
-        stackTTL = sum(totalsByCatagByStackId[stackId][catag] for catag in totalsByCatagByStackId[stackId])
-        for j, catag in enumerate(totalsByCatagByStackId[stackId]): # assuming ordered
-            if i == 0:
-                masterCatagOrder.append(catag)
-            elif masterCatagOrder[j] != catag:
-                raise Exception("Stacks passed in don't have categories in same order")
-            # str(catag) as JSON dump turns lvalue ints to strs
-            if usePerc == False:
-                data[catag].append(totalsByCatagByStackId[stackId][str(catag)])
-            else:
-                data[catag].append(100 * totalsByCatagByStackId[stackId][str(catag)]/stackTTL)
-                                 
-    sns.set() 
-    fig, ax = plt.subplots(figsize=(len(index) * 1.5, 5)) # NEED TO CALC FROM HEIGHT DIFFERENCE TODO
-    if usePerc == False:
-        ax.yaxis.set_major_formatter(tick.FuncFormatter(reformat_large_tick_values))
-
-    width = .75 # ie/ leave .125 each side between bars
+    data = defaultdict(list)
+    dataPercents = defaultdict(list)
+    for vista in sorted(commonTotals, key=lambda x: sum(commonTotals[x][k] for k in commonTotals[x])):
+        index.append(vista)
+        vistaTTL = sum(commonTotals[vista][cnt] for cnt in commonTotals[vista])
+        for cnt in range(4, 0, -1):
+            data[cnt].append(commonTotals[vista][cnt])
+            dataPercents[cnt].append(100 * commonTotals[vista][cnt]/vistaTTL)
+                 
+    def makeBasicBalanceAX(dta, index):
+        sns.set() 
+        fig, ax = plt.subplots()
+        width = .75 # the width of the bars: can also be len(x) sequence
+        # Lay down 4's, then 3's, the 2's ie/ X by X
+        bar4 = ax.bar(index, dta[4], width)
+        ttlsSoFar = dta[4][:]
+        bar3 = ax.bar(index, dta[3], width, bottom=ttlsSoFar)
+        ttlsSoFar = [a + b for a, b in zip(ttlsSoFar, dta[3])]
+        bar2 = ax.bar(index, dta[2], width, bottom=ttlsSoFar) 
+        ttlsSoFar = [a + b for a, b in zip(ttlsSoFar, dta[2])]
+        bar1 = ax.bar(index, dta[1], width, bottom=ttlsSoFar, label=1) # alt to label
+        ttlsSoFar = [a + b for a, b in zip(ttlsSoFar, dta[1])]
+        # Two ways to relabel - use label= as above; the handles code here; legend map
+        # handles, labels = ax.get_legend_handles_labels()
+        # ax.legend(reversed(handles), reversed(labels), loc='upper right')
+        ax.legend((bar1[0], bar2[0], bar3[0], bar4[0]), ("NONE", 2, 3, "ALL"), fontsize=10)
+        ax.yaxis.get_major_ticks()[0].label1.set_visible(False) # remove '0'
+        return ax, ttlsSoFar, dta[4]
         
-    bars = []
-    xs = [-0.15, 0.85, 1.85, 2.85, 3.85, 4.85, 5.85]
-    ttlsSoFar = [0 for i in range(0, len(totalsByCatagByStackId))]
-    firstTTLs = None
-    def formatNumber(fnum):
-        num = round(fnum, 1)
-        if num % 1 == 0:
-            return int(num)
-        else:
-            return num
-    for i, catag in enumerate(data):
-        barn = ax.bar(index, data[catag], width, bottom=ttlsSoFar) # layer
-        bars.append(barn)
-        if usePerc: # % inside bar pieces if > 10%
-            for j, dp in enumerate(data[catag]):   
-                if dp < 10:
-                    continue
-                plt.text(xs[j], (ttlsSoFar[j] - 1) + dp/2, f'{formatNumber(dp)}%', fontsize=11) 
-        ttlsSoFar = [a + b for a, b in zip(ttlsSoFar, data[catag])] 
-        if i == 0:
-            firstTTLs = ttlsSoFar[:]
-    if not usePerc: # total on top of bar
-        for i, ttl in enumerate(ttlsSoFar):
-            dataStr = reformat_large_tick_values(ttlsSoFar[i], roundTo=2)
-            plt.text(xs[i], 1.01 * ttlsSoFar[i], dataStr, fontsize=10) 
-        # Only want text first if > 10%
-        if len(set(firstTTLs)) == 1 and firstTTLs[0]/ttlsSoFar[-1] > .1: 
-            noToShow = firstTTLs[0]
-            txt = plt.text(xs[len(totalsByCatagByStackId) - 1], round(firstTTLs[0] * 0.9), reformat_large_tick_values(firstTTLs[0], roundTo=2), fontsize=11)
-            
-    legendLoc = "upper left" if usePerc == False else "lower left" # as big % first on left while biggest on right for absolutes
-    ax.legend(reversed([barn[0] for barn in bars]), reversed(list(data.keys())), fontsize=9, loc=legendLoc) # ncol=len(bars) setting ncol will make legend horizontal
-    # Two ways to relabel - use label= as above; the handles code here; legend map
-    # handles, labels = ax.get_legend_handles_labels()
-    # ax.legend(reversed(handles), reversed(labels), loc='upper right')
-    ax.yaxis.get_major_ticks()[0].label1.set_visible(False) # remove '0'   
-    
-    # ax.set_ylabel("{} Count".format(dirName.capitalize()))
-    ax.set_title(title) # whatever passes as auto needs too many overloads
-            
+    ax, ttlsSoFar, inAllTotals = makeBasicBalanceAX(data, index)
+    ax.set_ylabel("{} Count".format(dirName.capitalize()))
+    ax.set_title("Common {} Count".format(dirName.capitalize()))
+    # not 0, 1, 2, 3 but .025 off so centered
+    offsetUp = round(ttlsSoFar[3]/130)
+    plt.text(-0.20, ttlsSoFar[0] + offsetUp, "{:,}".format(ttlsSoFar[0]), fontsize=10)
+    plt.text(0.80, ttlsSoFar[1] + offsetUp, "{:,}".format(ttlsSoFar[1]), fontsize=10)
+    plt.text(1.80, ttlsSoFar[2] + offsetUp, "{:,}".format(ttlsSoFar[2]), fontsize=10)
+    plt.text(2.80, ttlsSoFar[3] + offsetUp, "{:,}".format(ttlsSoFar[3]), fontsize=10)
+    if dirName == "user": # TODO: make work for patient
+        plt.text(2.80, inAllTotals[0] - (7 * offsetUp), "{:,}".format(inAllTotals[0]), fontsize=10)
+    plt.tight_layout()
+    # plt.show()
+    imgRefTTLs = flushPlots("{}CommonTotals".format(dirName), flushToDirectory)
+
+    # TODO: much better to calculate offsets of .text than be abs
+    ax, ttlsSoFar, inAllTotals = makeBasicBalanceAX(dataPercents, index)
+    ax.set_title("Common {} %".format(dirName.capitalize()))
+    if dirName == "user": # TODO: make work for patient with legend +
+        plt.text(-0.15, inAllTotals[0] - 5, "{}%".format(round(inAllTotals[0], 1)), fontsize=10)
+        plt.text(0.85, inAllTotals[1] - 5, "{}%".format(round(inAllTotals[1], 1)), fontsize=10)
+        plt.text(1.85, inAllTotals[2] - 5, "{}%".format(round(inAllTotals[2], 1)), fontsize=10)
+        plt.text(2.85, inAllTotals[3] - 5, "{}%".format(round(inAllTotals[3], 1)), fontsize=10)
     plt.tight_layout()
     
 # ####################### Sankey ####################
 
 """
-TODO:
-- % on outs (not absolutes)
-- put total into title if want ex/ Travel (X%)
-- size --- see if height or width changes make distance better (could use fuller names than BOI etc)
-- clean comment below
-- see if anything in https://flothesof.github.io/sankey-tutorial-matplotlib.html on angles etc and also on "other"
-------
-- add a sankey from https://towardsdatascience.com/sankey-diagram-basics-with-pythons-plotly-7a13d557401a (simple and flows better for the many of Travel!)
-                and
-https://towardsdatascience.com/visualizing-in-app-user-journey-using-sankey-diagrams-in-python-8373a7bb2d22 <------- could do TELER events ... completed transitions! ... show all event flows [for test sets too]
-
 https://het.as.utexas.edu/HET/Software/Matplotlib/api/sankey_api.html
 
 This better tut: https://flothesof.github.io/sankey-tutorial-matplotlib.html
@@ -580,29 +461,26 @@ FUTURE:
 NEXT GO TO IMPROVE SANKEY:
 - GITHUB made good normalized versions on top of matplotlib: https://github.com/anazalea/pySankey/blob/master/pysankey/sankey.py
 
-    outValues = [(1355, "PUG"), (680, "POR"), (615, "BOI"), (263, "SPO"), (37, "WCTY")]
-    plotOutOtherSankey(outValues, 22, "Other IFCs Placed")
+    toVISNValues = [(1355, "PUG"), (680, "POR"), (615, "BOI"), (263, "SPO"), (37, "WCTY")]
+    plotVISNSankey(toVISNValues, 22, "Other IFCs Placed")
 """
-def plotOutOtherSankey(outValues, toOtherTTL, fromLabel, title): 
+def plotVISNSankey(toVISNValues, toOtherTTL, fromLabel, title): 
     
     # Use seaborn style defaults, bg etc
     sns.set() 
     
-    outValues = sorted(outValues, key=lambda x: x[0], reverse=True) # big to small north
-    outTTL = sum(d[0] for d in outValues)
+    toVISNValues = sorted(toVISNValues, key=lambda x: x[0], reverse=True) # big to small north
+    toVISNTTL = sum(d[0] for d in toVISNValues)
     
-    if outTTL == 0 and toOtherTTL == 0:
-        raise Exception(f'Can only OutOther Sankey if there is at least one or other total for OUT or OTHER: {title}')
-    
-    # outDir = 1 if outTTL > toOtherTTL else -1
-    outDir = 1 # keeping one orientation (not differing with size)
-    # toOtherDir = 1 if outDir == -1 else -1 
+    # toVISNDir = 1 if toVISNTTL > toOtherTTL else -1
+    toVISNDir = 1 # keeping one orientation (not differing with size)
+    # toOtherDir = 1 if toVISNDir == -1 else -1 
     toOtherDir = -1
     
-    flows = [d[0] * -1 for d in outValues] 
-    labels = [d[1] for d in outValues]
-    orientations = [outDir for d in outValues]
-    pathlengths = [0.25 for d in outValues]
+    flows = [d[0] * -1 for d in toVISNValues] 
+    labels = [d[1] for d in toVISNValues]
+    orientations = [toVISNDir for d in toVISNValues]
+    pathlengths = [0.25 for d in toVISNValues]
 
     # Only show OTH is there is one!
     if toOtherTTL:
@@ -611,7 +489,7 @@ def plotOutOtherSankey(outValues, toOtherTTL, fromLabel, title):
         orientations.append(toOtherDir)
         pathlengths.append(0.25)
 
-    total = outTTL + toOtherTTL
+    total = toVISNTTL + toOtherTTL
     # labels.insert(0, fromLabel) - don't bother with From as known
     labels.insert(0, "")
     flows.insert(0, total)
@@ -651,16 +529,6 @@ Based on https://www.dataquest.io/blog/tutorial-time-series-analysis-with-pandas
 - matplotlib
 - sns (need more info)
 
-Consider weekday out -- 
-
-    from pandas.tseries.offsets import BDay  
-    isBusinessDay = BDay().onOffset
-    csv_path = 'C:\\Python27\\Lib\\site-packages\\bokeh\\sampledata\\daylight_warsaw_2013.csv'
-    dates_df = pd.read_csv(csv_path)
-    match_series = pd.to_datetime(dates_df['Date']).map(isBusinessDay)
-    dates_df[match_series]
-    Crude df[df.index.dayofweek < 5]
-
 TODO: Typer and More
 - Fill in gaps in Patient date created data
 - Feed to subtyping (needs to be more compact if its to keep dailies!)
@@ -684,10 +552,12 @@ sns' boxplots as no great monthly variation to showcase ...
         ax.set_xlabel('')
     plt.show()
     return
-    
-# TODO: allow end="" ie/ start:"" works or?
 """
-def plotDailies(dailies, title, colNameById, start='2020-01', end='2020-02'):
+
+"""
+See 1-20->2-20: Expected drop on holiday monday's (MLK on the 1/20; President's day on 2/17) as well as 1/1 which falls on a Wednesday.
+"""
+def plotDailies(dailies, dirName, vistaNamesPlus, start='2020-01', end='2020-02'):
 
     # Use seaborn style defaults, bg etc
     sns.set() 
@@ -696,24 +566,18 @@ def plotDailies(dailies, title, colNameById, start='2020-01', end='2020-02'):
     # Hone in further and add a marker and line style
     # ... must plot directly with matplotlib and not the facade as date
     # calculation for the date formatters differs a little and sets markers off
-    for colId in colNameById: # not 'columns' as > colId's in cols now
-        """
-        FIX ABOVE for user dir etc -- TODO not here
-        
-        if not re.match(r'\d+$', colId):
+    for sno in vistaNamesPlus: # not 'columns' as > sno's in cols now
+        if not re.match(r'\d+$', sno):
             continue # exclude ALL but VistAs ... no 'overall' or total users
-        """
-        dailiesLocCol = dailies.loc[start:end, colId]
-        legendLabel = f'{colNameById[colId]} {int(dailiesLocCol.min())}/{int(dailiesLocCol.median())}/{int(dailiesLocCol.max())}'
-        ax.plot(dailiesLocCol, marker='o', linestyle='-', label=legendLabel)
-    ax.set_title(title)
+        ax.plot(dailies.loc[start:end, sno], marker='o', linestyle='-', label=vistaNamesPlus[sno])
+    ax.set_title('{}-{} {} Addition'.format(re.sub(r'\-', '/', start), re.sub(r'\-', '/', end), dirName.capitalize()))
     ax.set_ylabel('Daily Addition')
     ax.xaxis.set_major_locator(mdates.WeekdayLocator(byweekday=mdates.MO))
     # Format x-tick labels as 3-letter month name and day number (strftime form)
     ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d')) # '%b %d'
     # ax.xaxis.grid(color='blue', alpha=0.5) -- work the axis color for 
     ax.legend()
-        
+    
     plt.tight_layout()
     
 """
@@ -723,26 +587,23 @@ def plotDailies(dailies, title, colNameById, start='2020-01', end='2020-02'):
 ...
 > Rolling window operations are another important transformation for time series data. Similar to downsampling, rolling windows split the data into time windows and and the data in each window is aggregated with a function such as mean(), median(), sum() ... unlike downsampling, where the time bins do not overlap and the output is at a lower frequency than the input, rolling windows overlap and “roll” along at the same frequency as the data, so the transformed time series is at the same frequency as the original time series.
 """
-def plot365RollingMean(dailies, title, colNameById, start="1987-01", end="2021-12"):
-        
+def plot365RollingMean(dailies, dirName, vistaNamesPlus):
+    
     # 365-day rolling mean 
-    _365DayRollingMeans = dailies.loc[start:end, list(colNameById)].rolling(window=365, center=True).mean()    
+    start, end = '1987-01', ''
+    _365DayRollingMeans = dailies.loc[start:, list(vistaNamesPlus)].rolling(window=365, center=True).mean()    
     sns.set() # makes bg and lines
     fig, ax = plt.subplots(figsize=(11, 6))
-    for colId in colNameById:
-        """
-        TODO: fix above for users etc
-        
+    for sno in vistaNamesPlus:
         if not re.match(r'\d+$', sno):
             continue # exclude ALL but VistAs ... no 'overall' or total users
-        """
-        ax.plot(_365DayRollingMeans[colId], label=colNameById[colId])
+        ax.plot(_365DayRollingMeans[sno], label=vistaNamesPlus[sno])
     # Set x-ticks to yearly interval, adjust y-axis limits, add legend and labels
     ax.xaxis.set_major_locator(mdates.YearLocator(2))
     ax.xaxis.set_major_formatter(mdates.DateFormatter('%y'))
     ax.legend()
     ax.set_ylabel('Added')
-    ax.set_title(title)
+    ax.set_title('Trends in {} Addition (365 day rolling)'.format(dirName.capitalize()));
     
     plt.tight_layout()
 
@@ -752,44 +613,23 @@ resample the data to monthly frequency, aggregating with sum totals instead of t
 Note: article used stacked for some subdata
     opsd_monthly[['Wind', 'Solar']].plot.area(ax=ax, linewidth=0)
 """
-def plotMonthlyResamples(dailies, title, colNameById, start='2018-01', end='2020-02'):
+def plotMonthlyResamples(dailies, dirName, vistaNamesPlus):
     
-    # TODO: add check to make sure end gives full month to play with from
-    # passed in data (ONLY CHECKING START NOW!)
+    monthlies = dailies[list(vistaNamesPlus)].resample('M').sum()
     
-    # TODO: may change to "end" and "timeBack" ala utility => easier to QA inside
-    # graph and no duplication logic <----- see timeBack in CSV prod util
-    # Must make sure that start is supported by data frame input and prep to resample
-    # a month back from asked for support date to avoid dip while also avoiding need
-    # to resample the whole of a large data collection
-    startDay = start
-    if len(startDay) == 4:
-        startDay = f'{startDay}-01-01'
-    elif len(startDay) == 7:
-        startDay = f'{startDay}-01'  
-    startDayDT = datetime.strptime(startDay, "%Y-%m-%d")
-    startDayLess1MTHDT = startDayDT - relativedelta(months=1)
-    startDayLess1MTH = datetime.strftime(startDayLess1MTHDT, "%Y-%m-%d")
-    firstRowDay = datetime.strftime(dailies.index[0].to_pydatetime(), "%Y-%m-%d")
-    if firstRowDay > startDayLess1MTH:
-        raise Exception(f'Can\'t monthly resample from start date less a month, {startDayLess1MTH}, as first data date, {firstRowDay}, comes after that')
-    # Resample going back a month to make sure won't start on downturn
-    dailies = dailies.loc[startDayLess1MTH:end]
-
-    monthlies = dailies[list(colNameById)].resample('M').sum()
-        
     sns.set() # makes bg and lines
     fig, ax = plt.subplots()
-    for colId in colNameById: 
+    start, end = '2018-01', '2020-02'
+    for snop in vistaNamesPlus: # snop as showing overall if present too (for User)
         # color='black', green orange red
-        ax.plot(monthlies.loc[start:end, colId], label=colNameById[colId])
+        ax.plot(monthlies.loc[start:end, snop], label=vistaNamesPlus[snop])
     # ax.xaxis.set_major_locator(mdates.YearLocator())
     # ax.xaxis.set_major_locator(mdates.WeekdayLocator(byweekday=mdates.MO))
     # Format x-tick labels as 3-letter month name and day number (strftime form)
     ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%y'))
     ax.legend()
     ax.set_ylabel('Monthly Total')
-    ax.set_title(title)
+    ax.set_title('{} Resampled Monthly (2018-20)'.format(dirName.capitalize()));
     
     plt.tight_layout()
     
@@ -815,72 +655,30 @@ TODO: add in plt.annotate('Notice something?', xy=('2014-01-01', 30), xytext=('2
 ... consider doing manuals like this on top of returned plots => save is a separate
 step?
 """
-def plotTotalsPerYear(dailies, title, colNameById, start="2015", end="2020", snosOnly=True):
-
-    # TODO: FIX THIS SO NO SNOS etc -- just column names
+def plotTotalsPerYear(dailies, dirName, vistaNamesPlus):
 
     # Make sure only get VistA (sno) columns, not the total user | patient
-    snos = dict((vnp, colNameById[vnp]) for vnp in colNameById if re.match(r'\d+$', vnp)) if snosOnly else colNameById
+    snos = dict((vnp, vistaNamesPlus[vnp]) for vnp in vistaNamesPlus if re.match(r'\d+$', vnp))
     
-    dailiesRange = dailies.loc[start:end]
-    annuals = dailiesRange[list(snos)].resample('A').sum()
+    annuals = dailies[list(snos)].resample('A').sum()
     annuals = annuals.set_index(annuals.index.year)
     annuals.index.name = 'Year'
 
     sns.set() # makes bg and lines    
-
-    ax = annuals.plot.bar()
     
-    ax.set_title(title)
+    ax = annuals.loc['2015':'2020'].plot.bar()
+    
+    ax.set_title('{} Annual Additions (2015-20)'.format(dirName.capitalize()))
     
     # The following resets pandas plot wrapper defaults (rotated xticks, SNO not name ...)
     ax.legend(snos.values()) # want PUG, SPO etc
     # plt.xticks([0, 1, 2, 3, 4, 5], ["2015", "16", "17", "18", "19", "2020"], rotation=0)
-    xtickLabels = [str(yr) if str(yr) in [start, end] else str(yr)[2:] for yr in range(int(start.split("-")[0]), int(end.split("-")[0])+1)]
-    ax.set_xticklabels(xtickLabels, rotation=0) # method 2
+    ax.set_xticklabels(["2015", "16", "17", "18", "19", "2020"], rotation=0) # method 2
     ax.set_xlabel("") # don't need to see "Year"
     ax.yaxis.get_major_ticks()[0].label1.set_visible(False) # remove '0'
     
     plt.tight_layout()
     
-# ############################## UTILS ########################
-
-def reformat_large_tick_values(tick_val, pos=0, roundTo=1):
-
-    # code below will keep 4.5M as is but change values such as 4.0M to 4M since that zero after the decimal isn't needed
-    def removeTrail0(no):
-        # make no into a string value
-        nos = str(no)
-        index_of_decimal = nos.find(".")
-        if index_of_decimal == -1:
-            return no    
-        value_after_decimal = nos[index_of_decimal+1]
-        if value_after_decimal == "0":
-            # remove the 0 after the decimal point since it's not needed
-            nos = nos[0:index_of_decimal]
-            return int(nos)
-        return no
-
-    """
-    Turns large tick values (in the billions, millions and thousands) such as 4500 into 4.5K and also appropriately turns 4000 into 4K (no zero after the decimal).
-    """
-    if tick_val >= 1000000000:
-        val = round(tick_val/1000000000, roundTo)
-        new_tick_format = f'{removeTrail0(val)}B'
-    elif tick_val >= 1000000:
-        val = round(tick_val/1000000, roundTo)
-        new_tick_format = f'{removeTrail0(val)}M'
-    elif tick_val >= 1000:
-        # val = round(tick_val/1000, roundTo)
-        # new_tick_format = f'remoteTrail0(val)K'
-        new_tick_format = f'{removeTrail0(round(tick_val, 1)):,}'
-    elif tick_val < 1000:
-        new_tick_format = f'{removeTrail0(round(tick_val, 1))}'
-    else:
-        new_tick_format = tick_val
-            
-    return new_tick_format
-    
 # ############################## TEST DRIVER ################################
     
 # TODO: retire/ deprecate so charts off
@@ -905,9 +703,9 @@ def main():
     plotCategoryBH(df, "IFCs vs Non IFCs", "visualEx")
     plt.show()
     
-    outValues = [(1355, "PUG"), (680, "POR"), (615, "BOI"), (263, "SPO"), (37, "WCT")]
-    print(f'Plot would be stored in {viz.svgPlotFile("placerOther", "plotOutOtherSankey")}')
-    plotOutOtherSankey(outValues, 22, "WWW", "Other IFCs Placed")
+    toVISNValues = [(1355, "PUG"), (680, "POR"), (615, "BOI"), (263, "SPO"), (37, "WCT")]
+    print(f'Plot would be stored in {viz.svgPlotFile("placerOther", "plotVISNSankey")}')
+    plotVISNSankey(toVISNValues, 22, "WWW", "Other IFCs Placed")
     plt.show()
         
 if __name__ == "__main__":
