@@ -105,7 +105,11 @@ https://en.wikipedia.org/wiki/Outlier (Tukey's fences)
 ie/ K times Quartile. Fences for box plots.
 """    
 def keyStats(values):
-    quartiles = statistics.quantiles(values, method="inclusive") # inc matched numpy
+    try: # tmp allow for < 3.8
+        quartiles = statistics.quantiles(values, method="inclusive") # inc matched numpy
+    except:
+        import numpy
+        quartiles = [numpy.percentile(values, 25), numpy.percentile(values, 50), numpy.percentile(values, 75)]
     quartileOne = quartiles[0] # numpy.percentile(values, 25)
     median = quartiles[1] # numpy.percentile(values, 50)
     quartileThree = quartiles[2] # numpy.percentile(values, 75)
@@ -224,6 +228,8 @@ Note: RF part ie/ 200 into DataRF will move out of here. Tmp index
 maker for any type (including RF types) will become one simple utility.
 
 TODO: go to lookup by IEN as simpler
+
+Note: not dynamic if primary_menu_option or creator or title or # props >= 18 (or 16?)
 """
 def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
 
@@ -266,6 +272,9 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
                 
         info = {"hasProps": sorted([key for key in resource if key not in set(["_id", "label", "type"])])}
         
+        if "initial" in resource: # ex/ SMITH, JOE M --> JMS etc
+            info["initial"] = resource["initial"]
+        
         if "creator" in resource:
             # Only v1.1 will even have this ie/ not V10 on
             if resource["creator"]["id"] != "200-0":
@@ -278,23 +287,49 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         if "title" in resource:
             info["title"] = resource["title"]["label"] # it's a pointer
             
+        if "sex" in resource:
+            info["sex"] = resource["sex"] # old fashioned gender
+            
         if "ssn" in resource:
             info["ssn"] = resource["ssn"]
             
         # used for lookup BEFORE SSN in setup. goes with subject_organizations etc
+        # ... comment says within VA, VA SSOi, secid is the unique field
         if "secid" in resource:
             info["secid"] = resource["secid"]
-            if "subject_organization" in resource and resource["subject_organization"] != "Department Of Veterans Affairs":
-                print("** WARNING: Expected Subject Org for sec id to always be VA!")
+            # MOST are Dept of Vet Affairs - some VHA
+            if "subject_organization" in resource:
+                if resource["subject_organization"] not in ["Department Of Veterans Affairs", "Veterans Health Administration"]:
+                    print("** WARNING: Expected Subject Org for sec id to always be VA! but {}".format(resource["subject_organization"]))
+                if resource["subject_organization"] == "Veterans Health Administration":
+                    info["is_subject_organization_vha"] = True
             
+        # leave termination_reason for props list (ie/ presence or not)
         if "termination_date" in resource:
             info["termination_date"] = resource["termination_date"]["value"]
-        if "termination_reason" in resource:
-            info["termination_reason"] = resource["termination_reason"] 
             
         if "network_username" in resource:
             info["network_username"] = resource["network_username"]   
             
+        # adupn == va.gov email ... just check for now
+        if "adupn" in resource and not re.search(r'va.gov$', resource["adupn"]):
+            print("** unexpected adupn - email - not va.gov: {}".format(resource["adupn"]))
+            if "email_address" in resource and resource["email_address"] != resource["adupn"]:
+                print("** unexpected adupn - doesn't match email_address property: {}".format(resource["adupn"], resource["email_address"]))            
+            
+        """
+        Latest entries can have unique_user_id BUT mainly == secid or is email.
+        
+        Issue with not taking as is maybe that intention to use for lookup may
+        be missed - no as obvious there as listed in has_props
+        """
+        if "unique_user_id" in resource and not re.sub(r' ', '', resource["unique_user_id"]) == "":
+            if re.search(r'\@', resource["unique_user_id"]):
+                if "email_address" in resource and resource["unique_user_id"] != resource["email_address"]:
+                    info["distinct_unique_user_id"] = resource["unique_user_id"]
+            elif ("secid" in info and resource["unique_user_id"] != info["secid"]) or "secid" not in info:
+                info["__distinct_unique_user_id"] = resource["unique_user_id"]
+                                            
         if "last_signon_date_time" in resource:
             info["last_signon_date_time"] = resource["last_signon_date_time"]["value"]
             
@@ -327,6 +362,8 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         if "delegated_options" in resource:
             dos = set()
             for delo in resource["delegated_options"]:
+                if "delegated_options" not in delo:
+                    continue # seen in 663
                 if not re.match(r'\d+$', delo["delegated_options"]["label"]):
                     dos.add(delo["delegated_options"]["label"])
             if len(dos):                   
@@ -379,6 +416,11 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
     
 # ############################ Reduce Patient ########################
 
+"""
+if "date_of_death" in info: # note: may or may not have date_entered_into_file too
+    isDeceased
+... 1/5 or more.
+"""
 def reduce2(stationNo, forceNew=False):
 
     jsnFl = TMPWORKING_LOCN_TEMPL.format(stationNo) + "patientReduction.json"
@@ -599,6 +641,7 @@ def reduce4(stationNo):
         pass
     else:
         return infosByIEN
+    print("First time through, creating institution reduction for {}".format(stationNo))
     infosByIEN = {}
     for resource in filteredResultIterator(stationNo, "4"):
         ien = resource["_id"].split("-")[1]
@@ -621,6 +664,8 @@ def reduce4(stationNo):
         # VISN, Parent Facility etc
         if "associations" in resource:
             for association in resource["associations"]:
+                if "parent_of_association" not in association:
+                    continue
                 atyp = association["associations"]
                 apred = re.sub(r' ', '_', atyp["label"].lower())
                 ref = association["parent_of_association"]
@@ -634,9 +679,9 @@ def reduce4(stationNo):
         if "identifier" in resource:
             for identifier in resource["identifier"]:
                 ityp = identifier["coding_system"]
-                ipred = re.sub(r' ', '_', ityp["label"].lower()) + "_id"
-                id_02 = identifier["id_02"]
-                red[ipred] = id_02
+                ipred = re.sub(r' ', '_', ityp.lower()) + "_id"
+                id02 = identifier["id__02"]
+                red[ipred] = id02
         # taxonomy_code - reduce to list
     json.dump(infosByIEN, open(jsnFl, "w"), indent=4)
     return infosByIEN
